 	list p=16f628
;**************************************************************
;*	PORTA: 	0 - LCD:D0/4
;*		1 < 400V feedback = 1,235;   400 V -- 9,4 MOhm -- 29 kOhm
;*		2 < 1,235V referenz
;*		3 - LCD:D1/5
;*		4 o (nicht verwendet)
;*		5 < Schalter "Durchschnittswert"
;*		6 - LCD:D2/6
;*		7 - LCD:D3/7
;*	PORTB:	0 - Klickeingang fr Abstandsmessung (nicht verwendet)
;*		1 - RS232-RX (nicht verwendet)
;*		2 - RS232-TX
;*		3 > PWM fuer Transverter
;*		4 > LCD:R/W
;*		5 > LCD:RS
;*		6 < Eingang fuer Zaehlpulse vom GMZ zum Timer1
;*		7 > LCD:E 
;**************************************************************
;
; sprut (zero) Bredendiek 04/2011
;
; Geigerzaehler
;
;
;
; Prozessor 16F628 o..
; Prozessor-Takt 4 MHz intern
;
; GMZ geht von 0,144 ... 1444 Mikrosievert (4200 ticks/s)
; 1444 uS = 1444000 nS = 0x1608A0 nS -> 24Bit
;
; Timer0 : Zeitbasis = 65536 us (15,26Hz)
; Timer1 : Zaehler fuer ticks, max 65535 in 4,2 s = 15600 ticks/s = 5380 Mikrosievert
; Timer2 : PWM
;
; Berechnung nach ticks pro Zeiteinheit:
; Mikrosievert = kliks/sekunde / 2.9
;              = klicks/4.2s   / 12
; Nanosievert  = klicks/4.2s   * 82
;
;
; 400-V Stabilisierung
; Comparator
; Mode: CM2:CM0 = 101
; benoetigt RA1&2
; koennte interrupt ausloesen, aber besser ich schaue in festen Zeitabstaenden nach
;
; PWM steuert Spannungsstabilisierung 
;
;**************************************************************

; Includedatei fr den 16F628 einbinden
	#include <P16f628.INC>
;	#INCLUDE <MATH16.INC>    ; PIC16 math library definitions, belegt 0x20 bis 0x4B

	ERRORLEVEL      -302    ; SUPPRESS BANK SELECTION MESSAGES

; Configuration festlegen:
; Power on Timer, kein Watchdog, interner-Oscillator, kein Brown out, kein LV-programming
	__CONFIG	_MCLRE_OFF & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_NOCLKOUT & _BODEN_OFF &_LVP_OFF


;**************************************************************
; Variablen festlegen

; Speicherbereich fr Variablen
; Bank0: 0x20 .. 0x7F ; 96 Byte
; Bank1: 0xA0 .. 0xEF ; 80 Byte
; Bank2:0x120 ..0x14F ; 46 Byte
; 0x70..0x7F = 0xF0..0xFF = 0x170..0x17F

; Ringpuffer 1	A0..BF	16 x 2 byte
; Ringpuffer 2	C0..EF	16 x 3 byte

Temp		equ	0x4C

LcdStatus	equ	0x4D	;
LcdDaten	equ	0x4E	;

RBPointer	equ	0x4F	; pointer im ringpuffer A0..BF
PWM		equ	0x50	; Tastverhaeltnis 

SievL		equ	0x51	;
SievH		equ	0x52	;
SievU		equ	0x53	;
siTimer		equ	0x54	;

Fehler		equ	0x55	; Fehlerregister fr Mathematik
; Dezimalstellen
HdHM		equ	0x56	; 100Millionen
HdZM		equ	0x57	; ZehnMillionen
HdM		equ	0x58	; Millionen
HdHT		equ	0x59	; 100Tausender
HdZT		equ	0x5A	; ZehnTausender
HdT		equ	0x5B	; Tausender
HdH		equ	0x5C	; Hunderter
HdZ		equ	0x5D	; Zehner
HdE		equ	0x5E	; Einer
HdX		equ	0x5F	; Puffer fr eine Dezimalstelle

;16 Bit Rechenregister
f0		equ	0x60	; 
f1		equ	0x61	; 
f2		equ	0x62	; 
;16 Bit Rechenregister
xw0		equ	0x63	; 
xw1		equ	0x64	;
xw2		equ	0x65	;  
;16 Bit Rechenregister
g0		equ	0x66	; 
g1		equ	0x67	; 
g2		equ	0x68

EEWriteData	equ	0x69
loops		equ	0x6A
loops2		equ	0x6B
T0		equ	0x6C
T1		equ	0x6D
T2		equ	0x6E
Zeichen		equ	0x6F	; Zeichen von RS232

; ab hier in allen Baenken
w_copy		equ	0x70	; nur fr INT
s_copy		equ	0x71	; nur fr INT
p_copy		equ	0x72	; nur fr INT

Flags		equ	0x73	;

oldSievL	equ	0x74	;
oldSievH	equ	0x75	;
oldSievU	equ	0x76	;

CHARCTR		equ	0x78	; fuer CGRAM-Routinen		
TEMP		equ	0x79
pos		equ	0x7A	; Displayposition fuer Balkengrafik

RB2Pointer	equ	0x7B	; pointer im 2. ringpuffer C0..EF


TMPU		equ	0x7D	; !! in allen baenken
TMPH		equ	0x7E	; !! in allen baenken
TMPL		equ	0x7F	; !! in allen baenken


; Konstanten festlegen
#define LcdPort 	PORTA		; Datenbus des LCD (obere 4 Bit)
#define LcdRs		PORTB,5
#define LcdRw		PORTB,4
#define LcdE		PORTB,7

#define	DCDC		PORTA,4
#define	AVG		PORTA,5		; Taste fuer Durchschnittswert

#define	DatenSindDa	Flags,7		; rs232 hat was empfangen
#define	MMok		Flags,6		; Ringpuffer bereit fuer Minutendurchschnitt
#define BatLow		Flags,5		; Betriebsspannung zu klein
#define	BatHigh		Flags,4		; Betriebsspannung zu hoch
#define	RundeRum	Flags,3		; 67 sekunden vorbei
#define	MMok2		Flags,2		; Ringpuffer2 bereit fuer 15-Minutendurchschnitt


Ini_opt		equ	B'10000100'	; Timer0 int 32:1, pull-up on

;**************************************************************
	org	0
	goto	Init


;**************************************************************
; die Interuptserviceroutine
; interrupt fuer load-pulse an RB0
; int fuer shift-clock an RB4

	org 	4 	
intvec
	movwf	w_copy		; w retten
	swapf	STATUS, w 	; STATUS retten
	clrf	STATUS
	movwf	s_copy		;
	movf	PCLATH, W
	movwf	p_copy
	clrf	PCLATH		; Bank 0 

	btfss	INTCON,T0IF
	goto	Int_notT0
	bcf	INTCON,T0IF
	; Timer0 kommt alle 65,5 ms (15,26 Hz)
	; incrementiere 24-Bit-Zeit
	incf	T0, f
	bnz	int_t042
	incf	T1, f
	bnz	int_t042
	incf	T2, f
int_t042
	; nach 64 int sind 4,2 Sekunden vorbei --> timer 1 auslesen
	incf	siTimer,f
	btfss	siTimer,6	; >=64
	goto	Int_end
	; Timer1 auslesen
	clrf	siTimer
	bcf	T1CON,TMR1ON	; macht alles einfacher
	clrf	TMPU
	MOVF	TMR1H, W	; Read high byte
	MOVWF	TMPH
	MOVF	TMR1L, W	; Read low byte
	MOVWF	TMPL
	clrf	TMR1L
	clrf	TMR1H
	bsf	T1CON,TMR1ON	; machte alles einfacher

	call	SaveTicks	; im Ringpuffer ablegen
	call	ShowTicks	; TMPH-TMPL -> 2. LCD-Zeile
	call	Ticks2Sievert	; TEMPH-TEMPL -> SievU-SievH-SievL in nano-Sievert
	btfsc	AVG		; wie steht der Schalter?
	call	MinutenMittelwert	; in erster Minute ruft das Mittelwert

	call	ShowSiev	; SievU-SievH-SievL -> LCD
	call	ShowFlags
	call	RS232Siev

	call	PwmAdjust
	
Int_notT0
	goto	Int_end		; DEBUG DEBUG DEBUG DEBUG DEBUG DEBUG DEBUG DEBUG DEBUG DEBUG DEBUG DEBUG


Int_Rs
	;RS232-Empfnger-Interupt? 
	btfss	PIR1,RCIF 
	goto	Int_Rs2		; Interrupt kam von wo anders 
	;RS232-Empfnger-Interupt! 
	movfw	RCREG		; RS232-Register auslesen 
	movwf	Zeichen		; und in den Speicher nach 'Zeichen' schreiben 
	bsf	DatenSindDa	; Kennzeichen fr gltige Daten setzen
	bcf	PIR1,RCIF	; interrupt-Flag lschen 
	goto	Int_end
Int_Rs2

Int_end
	movf	p_copy, W
	movwf	PCLATH
	swapf	s_copy, w	; STATUS zurck
	movwf	STATUS 
	swapf	w_copy, f	; w zurck mit flags
	swapf	w_copy, w
	retfie


;**************************************************************
TxtGt	dt	"Gamma-Monitor",' '+80h		; RETLW
TxtV	dt	"V. 1.1  sprut",' '+80h		; RETLW
TxtT	dt	" TICKS   ",' '+80h		; RETLW
TxtT1	dt	" TICKS",' '+80h		; RETLW


;**************************************************************
; das sind die 8 Symbole, die in den CGRAM des Displays kopiert werden muessen
CGRAMTABLE
	MOVF	CHARCTR,W
	ADDWF	PCL,F		;TABLE FOR CUSTOM CGRAM
	;	movlw	0x00			; L
	;	CALL	OutLcdDaten
	;Durchschnitt , Tabellenoffset 0
	RETLW	B'00001'	;     *
	RETLW	B'01110'	;  ***
	RETLW	B'10011'	; *  **
	RETLW	B'10101'	; * * *
	RETLW	B'11001'	; **  *
	RETLW	B'01110'	;  *** 
	RETLW	B'10000'	; *    
	RETLW	B'00000'	; 
	;batterie leer , Tabellenoffset 8
	RETLW	B'00110'	;   ** 
	RETLW	B'01111'	;  ****
	RETLW	B'01001'	;  *  *
	RETLW	B'01001'	;  *  *
	RETLW	B'01001'	;  *  *
	RETLW	B'01001'	;  *  *
	RETLW	B'01111'	;  ****
	RETLW	B'00000'	;
	;batterie voll , Tabellenoffset 16
	RETLW	B'00110'	;   ** 
	RETLW	B'01111'	;  ****
	RETLW	B'01111'	;  ****
	RETLW	B'01111'	;  ****
	RETLW	B'01111'	;  ****
	RETLW	B'01111'	;  ****
	RETLW	B'01111'	;  ****
	RETLW	B'00000'	;
	;| , Tabellenoffset 24
	RETLW	B'10000'	; *
	RETLW	B'10000'	; *
	RETLW	B'10000'	; *
	RETLW	B'10000'	; *
	RETLW	B'10000'	; *
	RETLW	B'10000'	; *
	RETLW	B'10000'	; *
	RETLW	B'10000'	; *
	;|| , Tabellenoffset 32
	RETLW	B'11000'	; **
	RETLW	B'11000'	; **
	RETLW	B'11000'	; **
	RETLW	B'11000'	; **
	RETLW	B'11000'	; **
	RETLW	B'11000'	; **
	RETLW	B'11000'	; **
	RETLW	B'11000'	; **
	;||| , Tabellenoffset 40
	RETLW	B'11100'	; ***
	RETLW	B'11100'	; ***	
	RETLW	B'11100'	; ***
	RETLW	B'11100'	; ***	
	RETLW	B'11100'	; ***	
	RETLW	B'11100'	; ***
	RETLW	B'11100'	; ***
	RETLW	B'11100'	; ***
	;|||| , Tabellenoffset 48
	RETLW	B'11110'	; ****
	RETLW	B'11110'	; ****
	RETLW	B'11110'	; ****
	RETLW	B'11110'	; ****
	RETLW	B'11110'	; ****
	RETLW	B'11110'	; ****
	RETLW	B'11110'	; ****
	RETLW	B'11110'	; ****
	;|||| , Tabellenoffset 54
	RETLW	B'11111'	; *****
	RETLW	B'11111'	; *****
	RETLW	B'11111'	; *****
	RETLW	B'11111'	; *****
	RETLW	B'11111'	; *****
	RETLW	B'11111'	; *****
	RETLW	B'11111'	; *****
	RETLW	B'11111'	; *****
;**************************************************************

;**************************************************************
; Das Programm beginnt mit der Initialisierung
	org	0x100
Init
	; Ports grob einstellen
	clrf	INTCON  	 ; Interupt disable
	clrf	PORTA
	clrf	PORTB
	bsf	STATUS, RP0	; Bank 1
	bsf	PCON,OSCF	; 4 MHz
	movlw	Ini_opt     	; pull-up on; Timer0 32:1 interner Takt
	movwf	OPTION_REG 

	movlw	B'01000111'
	movwf	TRISB
	movlw	B'00100110'
	movwf	TRISA
	bcf	STATUS, RP0	; Bank 0

	call	ClearRAM	; 0x20..0x7F loeschen
	call	ClearBuffer	; ringpuffer loeschen
	call	ClearBuffer2	; ringpuffer2 loeschen


; Display
	call	InitLcd
	movlw	TxtGt	; Name
	call	Write	
	call	LcdZeile2
	movlw	TxtV	; Version
	call	Write	


; 16F628 Comparatoreingnge
	; ein Comparator; RA1&RA2
	; Ausgang ist das Bit  CMCON,C2OUT
	BSF	CMCON, CM0
	BCF	CMCON, CM1
	BSF	CMCON, CM2
	BCF	DCDC
	; RA1&RA2 sind input
	BSF	STATUS, RP0	; Bank1
	MOVLW	0x06		; RA1&RA2 sind input
	IORWF	TRISA, f
	BCF	STATUS,	RP0	; Bank 0
	; int vorbereiten
	CALL	Wait1		; 1ms delay
	MOVF	CMCON, f	; Read CMCON to end change condition
	BCF	PIR1,CMIF	; int-flag loeschen
	BSF	STATUS,RP0	; Bank 1
;	BSF	PIE1,CMIE	; Comp-Int erlauben
	BCF	STATUS,RP0	; Bank 0
	BSF	INTCON,PEIE	; peripheral interrupts erlauben
	;BSF	INTCON,GIE	; Global interrupt enable - das machen wir spaeter


; USART initialisieren 
	BSF	STATUS,RP0	; Bank1 
	MOVLW	0x20		; Sender: RS232 
	MOVWF	TXSTA		; 
	BCF	STATUS,RP0	; Bank 0 
	MOVLW	0x80		; UART einschalten, aber ohne Empfaenger 
	;MOVLW	0x90		; UART einschalten mit Empfnger: RS232 
	MOVWF	RCSTA		; 

	; USART Baudrate einstellen 
	BSF	STATUS,RP0	; Bank1 
	MOVLW	D'12'		; Set Baud rate 19,2 kBPS bei 4 MHz 
	MOVWF	SPBRG 
	BSF	TXSTA, BRGH	; BRGH=1 
	BCF	STATUS,RP0	; Bank 0 

	;USART Interrupts vorbereiten 
	BSF	STATUS,RP0	; Bank1 
	;BSF	PIE1,RCIE	; Enable receive interrupts 
	BCF	STATUS,RP0	; Bank 0 
	clrf	PIR1		; alle Interruptflags lschen 
	BSF	INTCON,PEIE	; Interrupts von Peripheriegerten erlauben 
	;BSF	INTCON,GIE	; generell Interrupts erlauben - kommt spaeter


; PWM initialisieren
	; 16F628 erzeugt an RB3 ein 6,6 kHz-Signal 
	; Tastverhltnis 50%
	; Prozessortakt:  4 MHz
	; 4 MHz / 4 / 150 = 6,6kHz
	; Vorteiler 1:1 und Timer2 einschalten
	BCF	T2CON,T2CKPS1	; Vorteiler 1:1
	BCF	T2CON,T2CKPS0	; Vorteiler 1:1
	BSF	T2CON,TMR2ON	; Timer2 ein
	; Frequenz auf 20 kHz einstellen
	BSF	STATUS,RP0	; Bank1
	MOVLW	D'150'	
	MOVWF	PR2		; 6,6 kHz
	BCF	STATUS,RP0	; Bank0
	; Tastverhltnis auf 1,2V einstellen 
	MOVLW	D'90'		; (150-90)*20mV = 1200mV
	MOVWF	CCPR1L	
	movwf	PWM		; merken
	; RB3/CCP1 auf Ausgang stellen
	BSF	STATUS,RP0	; Bank1
	BCF	TRISB, 3	; RB3: output=0
	BCF	STATUS,RP0	; Bank 0
	; PWM MODE mit CCP1 initialisieren
	CLRF	CCP1CON		; CCP1-Modus aus
	BSF	CCP1CON,CCP1M3	; CCP1-Modus PWM-Mode
	BSF	CCP1CON,CCP1M2
	call	Wait100		; warten bis sich Spannungen stabilisiert haben
	; PWM einregeln (um bis zu 0,9V rauf oder runter) in 1 Sekunde
	movlw	D'70'
	movwf	Temp
PwmInit2
	call	Wait20
	call	PwmAdjust
	decfsz	Temp, f
	goto	PwmInit2


; RB6 als Eingang fuer Timer1 fuer Pulse vom GMZ
	clrf	TMR1L
	clrf	TMR1H
	; 1:1,  externer takt, osc-off, no-sync
	movlw	B'00000111'
	movwf	T1CON


; Timer0 als Arbeitszyklus - loest Interrupt aus
	; 4 Hz / 4 / 256 / 256 = 15.26 Hz = 65,5 ms
	; 65,5ms x 64 = 4,2 s
	clrf	siTimer
	clrf	TMR0
	BSF	STATUS, RP0	; Bank1
	bcf	OPTION_REG,T0CS	; interer Takt
	bcf	OPTION_REG,PSA	; Vorteiler zum Timer0
	bsf	OPTION_REG,PS2	; Vorteiler 256:1
	bsf	OPTION_REG,PS1	; Vorteiler 256:1
	bsf	OPTION_REG,PS0	; Vorteiler 256:1
	BCF	STATUS, RP0	; Bank0
	bsf	INTCON, T0IE

	bsf	INTCON, GIE


loop


	goto	loop



;*********************************************************************
;* PwmAdjust
;* in Abhaengigkeit von  CMCON, C2OUT  wird das Tastverhaeltnis 
;* von CCP1 veraendert
;* Ziel ist eine GMZ-Spannung von 400V, 
;* dafuer muss die 3V-Bateriespannung auf etwa 1,2V herabgeregelt werden
;*
;* bei 3V Eingangsspannung :
;* 150 Stufen
;* 1 Stufe = 20mV
;* Uout = 3V/150 * (150-PWM)
;* 64..128 = 1,72V .. 0,44V
;*
;*********************************************************************
PwmAdjust
	bcf	BatLow
	bcf	BatHigh
	btfsc	CMCON, C2OUT
	goto	PwmInc

PwmDec				; Spannung erhoehen
	decf	PWM, f
	; <= 0x20 ?		; <40h = D'64' -> (150-64)*20mV = 1,72V
	movfw	PWM
	andlw	0xE0
	bnz	PwmOK
	incf	PWM, f
	bsf	BatLow
	return
	
PwmInc				; Spannung verringern
	incf	PWM, f
	; > 0x80 ?		; >80h = D'128' = (150-128)*20mV = 0,44V
	movfw	PWM
	andlw	0x80
	bz	PwmOK
	decf	PWM, f
	bsf	BatHigh
	return

PwmOK
	movfw	PWM
	MOVWF	CCPR1L		; 50% von 100
	return


;*********************************************************************
;* Ticks2Sievert
;* Umrechnung der klicks pro 4,2 Sekunden in Mikrosievert
;* max ist 15 mS = 15000 uS = 0x3A98
;* uS = kliks/sekunde * 2.9
;*    = kliks/4,2s * (2.9 * 4.2)
;*    = TEMP * 12.18
;*    = TEMP*8 + TEMP*4
;
;* nS = TEMP * 82
;* nS = TEMP * (64 + 16 + 2)
;*
;* Input : TEMPH-TEMPL
;* Outout: SievU-SievH-SievL in nano-Sievert
;*********************************************************************
Ticks2Sievert
	clrf	SievU
	movfw	TMPH
	movwf	SievH
	movfw	TMPL
	movwf	SievL

	; siev = TEMP x 64
	movfw	TMPH
	movwf	SievU
	movfw	TMPL
	movwf	SievH
	clrf	SievL
	bcf	STATUS,C
	rrf	SievU,f
	rrf	SievH,f
	rrf	SievL,f
	rrf	SievU,f
	rrf	SievH,f
	rrf	SievL,f

	;TEMP x 2
	bcf	STATUS,C
	rlf	TMPL,f
	rlf	TMPH,f
	rlf	TMPU,f

	; add
	; x64 + x2
	movf	TMPL,w		; low byte
	addwf	SievL,f 	; low byte add

	movf	TMPH,w 		; next byte
	btfsc	STATUS,C 	; skip to simple add if C was reset
	incfsz	TMPH,w 		; add C if it was set
	addwf	SievH,f 	; high byte add if NZ

	movf	TMPU,w 		; next byte
	btfsc	STATUS,C 	; skip to simple add if C was reset
	incfsz	TMPU,w 		; add C if it was set
	addwf	SievU,f 	; high byte add if NZ

	;TEMP x 2 x 8
	bcf	STATUS,C
	rlf	TMPL,f
	rlf	TMPH,f
	rlf	TMPU,f
	bcf	STATUS,C
	rlf	TMPL,f
	rlf	TMPH,f
	rlf	TMPU,f
	bcf	STATUS,C
	rlf	TMPL,f
	rlf	TMPH,f
	rlf	TMPU,f

	; add
	; (x64 + x2) + x8
	movf	TMPL,w		; low byte
	addwf	SievL,f 	; low byte add

	movf	TMPH,w 		; next byte
	btfsc	STATUS,C 	; skip to simple add if C was reset
	incfsz	TMPH,w 		; add C if it was set
	addwf	SievH,f 	; high byte add if NZ

	movf	TMPU,w 		; next byte
	btfsc	STATUS,C 	; skip to simple add if C was reset
	incfsz	TMPU,w 		; add C if it was set
	addwf	SievU,f 	; high byte add if NZ

	return


;*********************************************************************
;* Mittelwert
;* falls Sievert < 1024   dann Sievert:= 0.25 Sievert + 0.75 oldSievert
;* oldSievert := Sievert
;*
;* diese funktion wird nur fuer die ersten 67 Sekunden verwendet
;* danach kommt MinutenMittelwert zum Einsatz
;*
;* Input : SievU-SievH-SievL
;*         oldSievU-oldSievH-oldSievL
;* Output: SievU-SievH-SievL
;*         oldSievU-oldSievH-oldSievL
;*********************************************************************
Mittelwert

MittelRechne
	;oldSief := 0.75 oldSiev
	bcf	STATUS,C
	rrf	oldSievU,f
	rrf	oldSievH,f
	rrf	oldSievL,f

	movfw	oldSievU
	movwf	f2
	movfw	oldSievH
	movwf	f1
	movfw	oldSievL
	movwf	f0

	bcf	STATUS,C
	rrf	oldSievU,f
	rrf	oldSievH,f
	rrf	oldSievL,f

	movf	f0,w		; low byte
	addwf	oldSievL,f 	; low byte add
	movf	f1,w 		; next byte
	btfsc	STATUS,C 	; skip to simple add if C was reset
	incfsz	f1,w 		; add C if it was set
	addwf	oldSievH,f 	; high byte add if NZ
	movf	f2,w 		; next byte
	btfsc	STATUS,C 	; skip to simple add if C was reset
	incfsz	f2,w 		; add C if it was set
	addwf	oldSievU,f 	; high byte add if NZ

	; Siev := 0.25 Siev
	bcf	STATUS,C
	rrf	SievU,f
	rrf	SievH,f
	rrf	SievL,f
	bcf	STATUS,C
	rrf	SievU,f
	rrf	SievH,f
	rrf	SievL,f

	;Siev := 0.25 Siev + 0.75 oldSiev
	movf	oldSievL,w	; low byte
	addwf	SievL,f 	; low byte add
	movf	oldSievH,w 	; next byte
	btfsc	STATUS,C 	; skip to simple add if C was reset
	incfsz	oldSievH,w 	; add C if it was set
	addwf	SievH,f 	; high byte add if NZ
	movf	oldSievU,w 	; next byte
	btfsc	STATUS,C 	; skip to simple add if C was reset
	incfsz	oldSievU,w 	; add C if it was set
	addwf	SievU,f 	; high byte add if NZ

MittelBack
	movfw	SievU
	movwf	oldSievU
	movfw	SievH
	movwf	oldSievH
	movfw	SievL
	movwf	oldSievL
	return


;*********************************************************************
;* ClearRAM
;* Variablen der Bank0: 0x20 .. 0x7F loeschen
;*
;*********************************************************************
ClearRAM
	movlw	0x20
	movwf	FSR		; setzt den Zeiger auf Adresse A0h
	bcf	STATUS, IRP	; Bank 0 oder 1
CrLoop
	clrf	INDF		; lscht die aktuelle Speicherzelle
	incf	FSR, f		; erhhen des Adressen-Zeigers
	btfss	FSR, 7		; schon 80 erreicht?
	goto	CrLoop		; nein, die nchste lschen
	return


;*********************************************************************
;* ClearBuffer
;* Ringpuffer loeschen
;* Ringpuffer fuer 16 Werte a 2 byte = 32 byte = 0x20 byte 
;* 0xA0 .. 0xBF
;*
;*********************************************************************
ClearBuffer
	movlw	0xA0
	movwf	RBPointer
	movwf	FSR		; setzt den Zeiger auf Adresse A0h
	bcf	STATUS, IRP	; Bank 0 oder 1
CBLoop
	clrf	INDF		; lscht die aktuelle Speicherzelle
	incf	FSR, f		; erhhen des Adressen-Zeigers
	btfss	FSR, 6		; schon C0 erreicht?
	goto	CBLoop		; nein, die nchste lschen
 	return


;*********************************************************************
;* ClearBuffer2
;* Ringpuffer2 loeschen
;* Ringpuffer fuer 16 Werte a 3 byte = 48 byte = 0x30 byte 
;* 0xC0 .. 0xEF
;*
;*********************************************************************
ClearBuffer2
	movlw	0xC0
	movwf	RB2Pointer
	movwf	FSR		; setzt den Zeiger auf Adresse C0h	1100 0000
	bcf	STATUS, IRP	; Bank 0 oder 1
CB2Loop
	clrf	INDF		; lscht die aktuelle Speicherzelle
	incf	FSR, f		; erhhen des Adressen-Zeigers
	btfss	FSR, 4		; schon D0 oder F0 erreicht?
	goto	CB2Loop		; nein, die nchste lschen
	btfss	FSR, 5		; schon F0 erreicht?
	goto	CB2Loop		; nein, die nchste lschen
 	return


;*********************************************************************
;* SaveTicks
;* Speichern der Ticks im Ringpuffer
;* Ringpuffer fuer 16 Werte a 2 byte = 32 byte = 0x20 byte
;* 0xA0 .. 0xBF
;*
;* Input : TMPH-TMPL
;* maximal 99999 ticks
;*********************************************************************
SaveTicks
	movfw	RBPointer
	movwf	FSR	
	movfw	TMPH
	movwf	INDF
	incf	FSR,f
	movfw	TMPL
	movwf	INDF

	incf	RBPointer,f
	incf	RBPointer,f
	btfss	RBPointer, 6
	return
	movlw	0xA0		; Pufferende erreicht
	movwf	RBPointer
	bsf	MMok		; ringuffer ist befuellt
	bsf	RundeRum	; wieder 67 Sekunden vorbei
	return

;*********************************************************************
;* SaveTickSumme
;* Speichern der TickSumme im Ringpuffer2
;* Ringpuffer fuer 16 Werte a 3 byte = 48 byte = 0x30 byte
;* 0xC0 .. 0xEF
;*
;* Input : f2-f1-f0
;*********************************************************************
SaveTickSumme
	bcf	RundeRum

	movfw	RB2Pointer
	movwf	FSR	
	movfw	f2
	movwf	INDF

	incf	FSR,f
	movfw	f1
	movwf	INDF

	incf	FSR,f
	movfw	f0
	movwf	INDF

	incf	RB2Pointer,f	
	incf	RB2Pointer,f
	incf	RB2Pointer,f
	btfss	RB2Pointer, 4	; D0 oder F0 erreicht?
	return
	btfss	RB2Pointer, 5	; F0 erreicht?
	return
	movlw	0xC0		; Pufferende erreicht
	movwf	RB2Pointer
	bsf	MMok2		; ringuffer2 ist befuellt

	return


;*********************************************************************
;* Minutenmittelwert
;* 16 * 4.2s = 67,2s ~ 1 min
;* Ringpuffer fuer 16 Werte a 2 byte = 32 byte = 0x20 byte
;* 0xA0 .. 0xBF
;*
;* veraendert: f2,-f0, xw2-xw0
;* Output: SievU-SievH-SievL
;*********************************************************************
MinutenMittelwert
	btfss	MMok		; ist der Puffer bereit ? (67 Sekunden nach power on)
	goto	Mittelwert	; nein, noch den einfachen Mittelwert verwenden

	; Alle 16 Werte des Ringpuffers zusammenaddieren
	clrf	f0
	clrf	f1
	clrf	f2

	movlw	0xA0
	movwf	FSR		; setzt den Zeiger auf Adresse A0h
	bcf	STATUS, IRP	; Bank 0 oder 1
MMLoop
	clrf	xw2
	movfw	INDF		; lesen der aktuellen Zelle
	movwf	xw1
	incf	FSR, f		; erhhen des Adressen-Zeigers
	movfw	INDF		; lesen der aktuellen Zelle
	movwf	xw0
	call	Add24
	incf	FSR, f		; erhhen des Adressen-Zeigers
	btfss	FSR, 6		; schon C0 erreicht?
	goto	MMLoop		; nein, die nchste 

	; hier Eintrag der Summe in den zweiten Ringpuffer
	btfss	RundeRum
	goto	MMsiev
	call	SaveTickSumme	; wird alle 67 Sekunden gerufen

MMsiev
	btfsc	MMok2
	goto	M15mittelwert	; 18-Minutendurchschnitt ermitteln

	; nun die summe mit 5,1 Nanosievert multiplizieren
	movfw	f2
	movwf	xw2
	movfw	f1
	movwf	xw1
	movfw	f0
	movwf	xw0
	bcf	STATUS,C
	rlf	f0, f
	rlf	f1, f
	rlf	f2, f
	bcf	STATUS,C
	rlf	f0, f
	rlf	f1, f
	rlf	f2, f
	call	Add24

	movfw	f2
	movwf	SievU
	movfw	f1
	movwf	SievH
	movfw	f0
	movwf	SievL
	return

;*********************************************************************
;* M15mittelwert
;* 16 * 4.2s *16 ~ 18 min
;* Ringpuffer fuer 16 Werte a 3 byte = 48 byte = 0x30 byte
;* 0xC0 .. 0xEF
;*
;* veraendert: f2,-f0, xw2-xw0
;* Output: SievU-SievH-SievL
;*********************************************************************
M15mittelwert

	; Alle 16 Werte des Ringpuffers2 zusammenaddieren
	clrf	f0
	clrf	f1
	clrf	f2

	movlw	0xC0
	movwf	FSR		; setzt den Zeiger auf Adresse A0h
	bcf	STATUS, IRP	; Bank 0 oder 1
M15Loop
	movfw	INDF		; lesen der aktuellen Zelle
	movwf	xw2

	incf	FSR, f		; erhhen des Adressen-Zeigers
	movfw	INDF		; lesen der aktuellen Zelle
	movwf	xw1

	incf	FSR, f		; erhhen des Adressen-Zeigers
	movfw	INDF		; lesen der aktuellen Zelle
	movwf	xw0

	call	Add24
	incf	FSR, f		; erhhen des Adressen-Zeigers

	btfss	FSR, 4		; schon D0 oder F0 erreicht?
	goto	M15Loop		; nein, die nchste 
	btfss	FSR, 5		; schon F0 erreicht?
	goto	M15Loop		; nein, die nchste 

	; Summe steht nun in f2-f1-f0

	; nun die summe mit 5,1 Nanosievert multiplizieren
	movfw	f2
	movwf	xw2
	movfw	f1
	movwf	xw1
	movfw	f0
	movwf	xw0
	bcf	STATUS,C
	rlf	f0, f
	rlf	f1, f
	rlf	f2, f
	bcf	STATUS,C
	rlf	f0, f
	rlf	f1, f
	rlf	f2, f
	call	Add24

	; nun die summe durch 16 dividieren
	bcf	STATUS,C
	rrf	f2, f
	rrf	f1, f
	rrf	f0, f
	bcf	STATUS,C
	rrf	f2, f
	rrf	f1, f
	rrf	f0, f
	bcf	STATUS,C
	rrf	f2, f
	rrf	f1, f
	rrf	f0, f
	bcf	STATUS,C
	rrf	f2, f
	rrf	f1, f
	rrf	f0, f

	movfw	f2
	movwf	SievU
	movfw	f1
	movwf	SievH
	movfw	f0
	movwf	SievL
	return


;*********************************************************************
;* ShowTicks
;* Ausgabe der Ticks am LCD
;* 
;*
;* Input : TMPH-TMPL
;* maximal 99999 ticks
;*********************************************************************
ShowTicks
	call	LcdZeile2

	movfw	TMPL
	movwf	f0
	movfw	TMPH
	movwf	f1
	clrf	f2
	call	Hex2Dez16	; Dezimal-Wandlung

	; Anzeige
	movfw	HdZT
	call	LcdAscii
	movfw	HdT
	call	LcdAscii
	movlw	' '
	call	OutLcdDaten
	movfw	HdH
	call	LcdAscii
	movfw	HdZ
	call	LcdAscii
	movfw	HdE
	call	LcdAscii

	movlw	TxtT	; TICKS
;	movlw	TxtT1	; TICKS  DEBUG
	call	Write

;	movlw	'#'		; bereit fuer 15-min-durchschnitt?
;	btfss	MMok2
;	movlw	' '
;	call	OutLcdDaten


	return


;*********************************************************************
;* ShowFlags
;* Ausgabe der Flags am LCD, max 2 Zeichen
;* 
;*
;*********************************************************************
ShowFlags
	movlw	' '
	call	OutLcdDaten

	; 1. Zeichen 
	movlw	'='		; kein Durchschnit sondern letzte 4 Sekunden
	btfss	AVG
	call	OutLcdDaten

	movlw	'.'		; Durchschnitt in der ersten Minute
	btfsc	MMok
	movlw	'*'		; Durchschnitt Minutendurchschnitt
	btfsc	MMok2
	movlw	0x00		; Durchschnitt 18-Minuten-Durchschnitt
	btfsc	AVG
	call	OutLcdDaten


	; 2. Zeichen
	;movlw	'b'		; Unterspannung
	movlw	0x01		; Unterspannung, Symbol: leere Batterie
	btfsc	BatLow
	call	OutLcdDaten

	movlw	'B'		; Ueberspannung
	btfsc	BatHigh
	call	OutLcdDaten

	; aufraeumen
	movlw	' '
	call	OutLcdDaten
	movlw	' '
	call	OutLcdDaten
	movlw	' '
	call	OutLcdDaten


	return


;*********************************************************************
;* ShowSiev
;* Ausgabe der Mikrosievert am LCD, 8-stellig
;* 
;*
;* Input : SievU-SievH-SievL
;* max. 99 999.999 uS = 10 mS
;* HdZM sollte immer 0 sein
;*********************************************************************
ShowSiev
	call	LcdZeile1

	movfw	SievL
	movwf	f0
	movfw	SievH
	movwf	f1
	movfw	SievU
	movwf	f2
	call	Hex2Dez24	; Dezimal-Wandlung

	; Anzeige
	movfw	HdZM
	call	LcdAscii
	movfw	HdM
	call	LcdAscii
	movlw	' '
	call	OutLcdDaten
	movfw	HdHT
	call	LcdAscii
	movfw	HdZT
	call	LcdAscii
	movfw	HdT
	call	LcdAscii
	movlw	'.'
	call	OutLcdDaten
	movfw	HdH
	call	LcdAscii
	movfw	HdZ
	call	LcdAscii
	movfw	HdE
	call	LcdAscii

	movlw	' '
	call	OutLcdDaten
	movlw	B'11100100'	;'u'
	call	OutLcdDaten
	movlw	'S'
	call	OutLcdDaten

	return


;*********************************************************************
;* Write
;* Ausgabe eines Strings der ab W+1 im Speicher steht
;* Note: Endekennzeichen ist Zeichen mit Bit 7 = 1
;*
;* Input : W zeigt auf String (RETLWs)
;*********************************************************************
Write 
	movwf	Temp		; Temp = Pointer
	decf	Temp, f
GoWrite	
	call	PclSub2		; Pointer erhhen und nchstes Zeichen lesen
	addlw	80h		; ist Bit 7 gesetzt? EOT
	btfsc	STATUS,C
	goto	OutLcdDaten	; letztes Zeichen
	andlw	7fh		; zeichen wieder herstellen
	call	OutLcdDaten	; Ausgabe
	goto	GoWrite	

PclSub2
	incf	Temp, F		; Pointer auf nchstes Zeichen
	movf	Temp, W		; Pointer nach W
	movwf	PCL		; Sprung zur Addresse auf die PCLATH,W zeigt


;*****************************************************
;+++LCD-Routinen**************************************
;*****************************************************
; LCD initialisieren
; es werden alle bits des PortB veraendert !

InitLcd
	;Wait for more than 15ms after VDD rises to 4.5V
	movlw	D'255'		; 250 ms Pause nach dem Einschalten
	movwf	loops	
	call	WAIT	

	bcf	LcdRw
	bcf	LcdRs
	
	movlw	B'00001001'	; 1
	movwf	LcdPort
	bsf	LcdE
	nop	
	bcf	LcdE
	
	;Wait for more than 4.1ms
	movlw	D'50'		; 50 ms Pause
	movwf	loops
	call	WAIT
	movlw	B'00001001'	; 2
	call	Control8Bit

	;Wait for more than 100s
	movlw	B'00001001'	; 3
	call 	Control8Bit

	movlw	B'00001000'	; 4 4-Bit-Interface aktivieren
	call 	Control8Bit

	movlw	B'00000001'	; lschen und cusor home
	call	OutLcdControl	
	movlw	B'00101000'	; 5 function set, 4-bit  2-zeilig,  5x7
	call	OutLcdControl	
	movlw	B'00001000'	; 6 display off
	call	OutLcdControl
	movlw	B'00000110'	; 7 entry mode, increment, disable display-shift
	call	OutLcdControl
	movlw	B'00000011'	; 8 cursor home, cursor home
	call	OutLcdControl
	movlw	B'00001100'	; 9 display on, Kursor aus
	call	OutLcdControl


;CGRAM Adresse auf 0x00 setzen
	MOVLW   B'00000000'
	CALL	SETCGADDR

;CGRAM mit den neuen Symbolen laden
	MOVLW	.0			;0: Durchschnitt
	CALL	CGRAMLOAD
	MOVLW	.8			;1: Batterie leer
	CALL	CGRAMLOAD
	MOVLW	.16			;2: CYR G
	CALL	CGRAMLOAD
	MOVLW	.24			;3: |
	CALL	CGRAMLOAD
	MOVLW	.32			;4: ||
	CALL	CGRAMLOAD
	MOVLW	.40			;5: |||
	CALL	CGRAMLOAD
	MOVLW	.48			;6: ||||
	CALL	CGRAMLOAD
	MOVLW	.56			;7: |||||
	CALL	CGRAMLOAD

;DDRAM ADRESSE SETZEN
	MOVLW   B'00000000'
	CALL	SETDDADDR  		; Kursor an den displayanfang setzen

	return


;*****************************************************
; LCD-Hilfsroutinen
LcdZeile1
	movlw	0x80
	goto	OutLcdControl

LcdZeile2
	movlw	0xC0
	goto	OutLcdControl

LcdAscii
	iorlw	'0'
	goto	OutLcdDaten


;*****************************************************
; ein Steuerbyte 8-bittig bertragen
Control8Bit
	movwf	LcdPort
	bcf	LcdRw
	bcf	LcdRs
	bsf	LcdE
	nop
	bcf	LcdE
	movlw	D'10'
	movwf	loops
	call 	WAIT
	return


;*****************************************************
; darauf warten, da das Display bereit zur Datenannahme ist
LcdBusy
        bsf     STATUS, RP0	; make Port B0,1,2,4 input
	;movlw	B'00010111'
	;iorwf   TRISB, f 
	movlw	B'11001001'
	iorwf   TRISA, f 

        bcf     STATUS, RP0
BusyLoop		
	bcf	LcdRs		; control
	bsf	LcdRw		; Lesen
	bsf	LcdE
	nop
	movf	LcdPort, w
	movwf	LcdStatus
	bcf	LcdE
	nop
	bsf	LcdE		; Enable
	nop
	bcf	LcdE
	btfsc	LcdStatus, 7	; teste MSB
	goto	BusyLoop
	bcf	LcdRw
        bsf     STATUS, RP0	; make Port B4..7 output
	movlw	B'00110110'
	andwf   TRISA, f    
        bcf     STATUS, RP0
	; beim KS0066U-Controller sollte man nach busy noch 10 Mikrosekunden warten
	; deshalb als failsave noch ein paar NOPs
	nop
	nop
	nop
	nop
	return	


;*****************************************************
; aus W ein Byte mit Steuerdaten zum Display bertragen
OutLcdControl
	movwf	LcdDaten
	call	LcdBusy
	bcf	LcdRw
	bcf	LcdRs

	; obere Haelfte nach 7630
	movfw	LcdDaten
	andlw	H'C0'
	btfsc	LcdDaten, 5
	iorlw	0x08
	btfsc	LcdDaten, 4
	iorlw	0x01

	movwf	LcdPort		; Hi-teil Daten schreiben
	bsf	LcdE
	nop
	bcf	LcdE		; Disable LcdBus

	; untere Haelfte nach 7630
	swapf	LcdDaten, w
	andlw	H'C0'
	btfsc	LcdDaten, 1
	iorlw	0x08
	btfsc	LcdDaten, 0
	iorlw	0x01

	movwf	LcdPort		; Lo-teil Daten schreiben
	bsf	LcdE
	nop
	bcf	LcdE		; Disable LcdBus
	return


;*****************************************************
; aus W ein Datenbyte zum Display bertragen
OutLcdDaten
	movwf	LcdDaten
	call	LcdBusy

	; obere Haelfte nach 7630
	movfw	LcdDaten
	andlw	H'C0'
	btfsc	LcdDaten, 5
	iorlw	0x08
	btfsc	LcdDaten, 4
	iorlw	0x01

	movwf	LcdPort		; Hi-teil Daten schreiben
	bsf	LcdRs		; Daten
	bcf	LcdRw		; schreiben
	bsf	LcdE		; Enable LcdBus
	nop
	bcf	LcdE		; Disable LcdBus

	; untere Haelfte nach 7630
	swapf	LcdDaten, w
	andlw	H'C0'
	btfsc	LcdDaten, 1
	iorlw	0x08
	btfsc	LcdDaten, 0
	iorlw	0x01

	movwf	LcdPort		; Lo-teil Daten schreiben
	bsf	LcdRs		; Daten
	bcf	LcdRw		; schreiben
	bsf	LcdE
	nop
	bcf	LcdE		; Disable LcdBus	
	bcf	LcdRs		;
	return


;**************************************************************
;* RS232-Ausgabe von siev
;*
;**************************************************************
RS232Siev
	movfw	HdZM
	call	RS232Ascii
	movfw	HdM
	call	RS232Ascii
	movlw	' '
	call	RS232TX
	movfw	HdHT
	call	RS232Ascii
	movfw	HdZT
	call	RS232Ascii
	movfw	HdT
	call	RS232Ascii
	movlw	'.'
	call	RS232TX
	movfw	HdH
	call	RS232Ascii
	movfw	HdZ
	call	RS232Ascii
	movfw	HdE
	call	RS232Ascii
	movlw	' '
	call	RS232TX
	movlw	'u'
	call	RS232TX
	movlw	'S'
	call	RS232TX

	movlw	0x0D
	call	RS232TX
	movlw	0x0A
	call	RS232TX
	return


;**************************************************************
;* RS232-Ausgabe eines Zeichens
;*
;**************************************************************
; Zeichen aus W nach RS232 ausgeben
RS232Ascii
	iorlw	'0'

RS232TX
	btfss	PIR1,TXIF	; ist Sender leer ? 
	goto	RS232TX		; nein, noch nicht leer
	movwf	TXREG		; w in den RS232-Sender schreiben 
	return


;**************************************************************
;+++Mathematik-Routinen ***************************************

;**************************************************************
; 24 Bit Subtraktion, bei berlauf (neg. Ergebnis) ist C gesetzt
Sub24				; 24 bit f:=f-xw   calc=xw cnt=f
	clrf	Fehler		; extraflags lschen

	movf    xw0, w		; f0=f0-xw0
	subwf   f0, f

	btfsc   STATUS,C
	goto    sb0
	movlw   0x01		; borgen von f1
	subwf   f1, f

	btfsc   STATUS,C
	goto    sb0
	subwf   f2    ,f        ; borgen von f2

	btfss   STATUS,C
	bsf	Fehler, C	; unterlauf

sb0     movf    xw1, w		; f1=f1-xw1
	subwf   f1, f

	btfsc   STATUS,C
	goto    sb1
	movlw   0x01		; borgen von f2
	subwf   f2    ,f

        btfss   STATUS,C
	bsf	Fehler, C	; Unterlauf

sb1     movf    xw2,w		; f2=f2-xw2
        subwf   f2, f

        btfss   STATUS,C
       	bsf	Fehler, C	; Unterlauf

	bcf	STATUS, C
	btfsc	Fehler, C
	bsf	STATUS, C
	return


;**************************************************************
;24 bit Adition, C-Flag bei berlauf gesetzt
Add24 				; 24-bit add: f = f + xw
	movf	xw0,w		; low byte
	addwf	f0,f 		; low byte add

	movf	xw1,w 		; next byte
	btfsc	STATUS,C 	; skip to simple add if C was reset
	incfsz	xw1,w 		; add C if it was set
	addwf	f1,f 		; high byte add if NZ
	
	movf	xw2,w		; high byte 
	btfsc	STATUS,C	; berspringe falls C nicht gesetzt 
	incfsz	xw2,w		; addiere C falls gesetzt 
	addwf	f2,f		; next byte add wenn  NZ 

	return 			; fertig


;**************************************************************
;primitive 24 bit Division 	f:= f / xw
; die Langsamkeit dieser Routine verursacht Displayflackern bei hohen Drehzahlen
Div24
	clrf	g0
	decf	g0, f
	clrf	g1
	decf	g1, f
	clrf	g2
	decf	g2, f
Div24Loop
	incf	g0, f
	btfsc	STATUS, Z
	incf	g1, f
	btfsc	STATUS, Z
	incf	g2, f
	call	Sub24		;
	btfss	STATUS, C	;berlauf
	goto	Div24Loop	;Stelle 1 mehr
	movfw	g0
	movwf	f0
	movfw	g1
	movwf	f1
	movfw	g2
	movwf	f2
	return


;**************************************************************
; positiven 16 Bit Wert (f1,f0) in dezimal umwandeln
;
;       10.000.000 = 98 96 80 h
;        1.000.000 = 0F 42 40 h
;
;          100.000 = 01 86 A0 h uS
;           10.000 = 00 27 10 h
;            1.000 = 00 03 E8 h
;
;              100 = 00 00 64 h nS
;               10 = 00 00 0A h
;                1 = 00 00 01 h
Hex2Dez24			; 24-bit(f2,f1,f0) in 8-stellen BCD (ZM, M, HT, ZT, T, H, Z, E)	
	movlw	0x98		; 10.000.000 = 98 96 80 h
	movwf	xw2
	movlw	0x96
	movwf	xw1
	movlw	0x80
	movwf	xw0
	clrf	HdX
	decf	HdX, f
HdLoop1
	incf	HdX, f
	call	Sub24		;
	btfss	STATUS, C	;berlauf
	goto	HdLoop1		;Stelle 1 mehr
	call	Add24
	movfw	HdX
	movwf	HdZM

	movlw	0x0F		; 1.000.000 = 0F 42 40 h
	movwf	xw2
	movlw	0x42
	movwf	xw1
	movlw	0x40
	movwf	xw0
	clrf	HdX
	decf	HdX, f
HdLoop2
	incf	HdX, f
	call	Sub24		;
	btfss	STATUS, C	;berlauf
	goto	HdLoop2		;Stelle 1 mehr
	call	Add24
	movfw	HdX
	movwf	HdM

	movlw	0x01		; 100.000 = 01 86 A0 h
	movwf	xw2
	movlw	0x86
	movwf	xw1
	movlw	0xA0
	movwf	xw0
	clrf	HdX
	decf	HdX, f
HdLoop3
	incf	HdX, f
	call	Sub24		;
	btfss	STATUS, C	;berlauf
	goto	HdLoop3		;Stelle 1 mehr
	call	Add24
	movfw	HdX
	movwf	HdHT

Hex2Dez16
	movlw	0x00		; 10.000 = 00 27 10 h
	movwf	xw2
	movlw	0x27
	movwf	xw1
	movlw	0x10
	movwf	xw0
	clrf	HdX
	decf	HdX, f
HdLoop4
	incf	HdX, f
	call	Sub24		;
	btfss	STATUS, C	;berlauf
	goto	HdLoop4		;Stelle 1 mehr
	call	Add24
	movfw	HdX
	movwf	HdZT

	movlw	0x00		; 1.000 = 00 03 E8 h
	movwf	xw2
	movlw	0x03
	movwf	xw1
	movlw	0xE8
	movwf	xw0
	clrf	HdX
	decf	HdX, f
HdLoop5
	incf	HdX, f
	call	Sub24		;
	btfss	STATUS, C	;berlauf
	goto	HdLoop5		;Stelle 1 mehr
	call	Add24
	movfw	HdX
	movwf	HdT

	movlw	0x00		; 100 = 00 00 64 h
	movwf	xw2
	movwf	xw1
	movlw	0x64
	movwf	xw0
	clrf	HdX
	decf	HdX, f
HdLoop6
	incf	HdX, f
	call	Sub24		;
	btfss	STATUS, C	;berlauf
	goto	HdLoop6		;Stelle 1 mehr
	call	Add24
	movfw	HdX
	movwf	HdH

	movlw	0x00		; 10 = 00 00 0A h
	movwf	xw2
	movwf	xw1
	movlw	0x0A
	movwf	xw0
	clrf	HdX
	decf	HdX, f
HdLoop7
	incf	HdX, f
	call	Sub24		;
	btfss	STATUS, C	;berlauf
	goto	HdLoop7		;Stelle 1 mehr
	call	Add24
	movfw	HdX
	movwf	HdZ

	movfw	f0
	movwf	HdE
	return


;*****************************************************
; Es folgen die Routinen fue CGRAM
;*****************************************************

;*****************************************************
;SET CGRAM TEST ROUTINE
;IN: W=CGRAM LOCATION
;DESTROYS; W, CHARCTR
;*****************************************************
SETCGADDR
	bcf	LcdRs 		; INSTR REG
	bcf	LcdRw		; WRITE
	ANDLW	B'00111111'	; CGRAM CHOPOFF
	IORLW	B'01000000' 	; CGRAM AREA
	CALL	OutLcdControl
	RETURN


;*****************************************************
;SET DDRAM TEST ROUTINE
;IN: W=DDRAM LOCATION
;DESTROYS; W, CHARCTR
;*****************************************************
SETDDADDR
	bcf	LcdRs 		; INSTR REG
	bcf	LcdRw		; WRITE
	ANDLW	B'01111111'	; DDRAM CHOPOFF
	IORLW	B'10000000' 	; DDRAM AREA
	CALL	OutLcdControl
	RETURN


;*****************************************************
;CGRAM CHARACTER LOAD ROUTINE
;IN: W=NUMBER
;DESTROYS; W, CHARCTR
;*****************************************************
CGRAMLOAD
	BSF	LcdRs 		; INSTR REG
	bcf	LcdRw		; WRITE
	MOVWF	CHARCTR		; CHARACTER COUNTER
	MOVLW	0X08
	MOVWF	TEMP		; 8 BYTES TO SEND
			
CGRAMSUB1
	CALL	CGRAMTABLE	;LOAD A CHAR FROM ROM TBL
	CALL	OutLcdDaten
	INCF	CHARCTR,F	;INC CHAR COUNTER
	DECFSZ	TEMP,F
	GOTO	CGRAMSUB1
	RETURN


;*****************************************************************	
;Zeitverzgerung um loops * 1 ms *********************************
;*****************************************************************
; universelle Warteschleife, sowas braucht man fter
; z.B. fr die LCD-Initialisierung
; in loops wird die Wartezeit in Millisekunden bergeben
;
; 16 MHz externer Takt bedeutet 4 MHz interner Takt
; also dauert 1 ms genau 4000 Befehle
; 250 Schleifen a 16 Befehle sind 2500 Befehle = 1 ms

; 10 MHz externer Takt bedeutet 2,5 MHz interner Takt
; also dauert 1 ms genau 2500 Befehle
; 250 Schleifen a 10 Befehle sind 2500 Befehle = 1 ms

; 8 MHz externer Takt bedeutet 2 MHz interner Takt
; also dauert 1 ms genau 2000 Befehle
; 200 Schleifen a 10 Befehle sind 1000 Befehle = 1 ms

; 4 MHz externer Takt bedeutet 1 MHz interner Takt
; also dauert 1 ms genau 1000 Befehle
; 100 Schleifen a 10 Befehle sind 1000 Befehle = 1 ms
Wait1
	movlw	D'1'		; 1 ms Pause 
	movwf	loops	
	goto	WAIT
Wait20
	movlw	D'20'		; 20 ms Pause 
	movwf	loops	
	goto	WAIT
Wait100
	movlw	D'100'		; 100 ms Pause 
	movwf	loops	
	goto	WAIT
Wait250
	movlw	D'250'		; 250 ms Pause
	movwf	loops	

WAIT
top
	movlw   .100           ; timing adjustment variable (1ms)  4 MHz
        movwf   loops2
top2    nop                    ; warten und nichts tun
        nop
        nop
        nop
	nop
        nop
        nop

        decfsz  loops2, F      ; innere Schleife fertig?
        goto    top2           ; nein, noch mal rum
                               ;
        decfsz  loops, F       ; uere Schleife fertig?
        goto    top            ; nein, noch mal rum
        retlw   0              ; FERTIG und return


;*****************************************************
;+++EEPROM-Routinen fr 16F628************************
;*****************************************************
; 16F628 hat 128 EEPROM-Zellen

; lesen der Zelle ^W nach W
EERead
	BSF  	STATUS, RP0 	; EEADR liegt in der Bank 1 
 	MOVWF 	EEADR 		; schreibe die Adresse in EEADR 
	BSF  	EECON1, RD 	; EEPROM Leseproze starten 
	MOVF 	EEDATA, W 	; Die Daten der EEPROM Zelle nach W kopieren 
	BCF 	STATUS, RP0 	; Bank 0 
	return

; schreibe EEWriteData in Zelle ^W
EEWrite
	BSF 	STATUS, RP0 	; EEADR und EEDATA liegen in der Bank 1 
	MOVWF 	EEADR 		; Die Zelle @W soll beschrieben werden 
	BCF 	STATUS, RP0
	MOVFW 	EEWriteData	; EEWritedata liegt in Bank 0
	BSF 	STATUS, RP0
	MOVWF 	EEDATA 		; EEWriteData wollen wir schreiben 
	BSF 	EECON1, WREN 	; nun ist Schreiben erlaubt 

; Die folgenden 5 Zeilen mssen genau so im Code stehen!!! 
	MOVLW 	055h
	MOVWF 	EECON2 		; schreibe 55h nach EECON2 
	MOVLW 	0AAh
	MOVWF 	EECON2 		; schreibe AAh nach EECON2 
	BSF 	EECON1, WR 	; starte den Schreibzyklus 

EEWrite1			; warten das write fertig ist
	btfsc	EECON1, WR
	goto	EEWrite1
	BCF 	STATUS, RP0 	; Bank 0 
	return


;**************************************************************

	end